@@ -56,20 +56,20 @@
 #' }
 #' @export
 sd_database <- function(
-    host       = NULL,
-    dbname     = NULL,
-    port       = NULL,
-    user       = NULL,
-    table      = NULL,
-    password   = Sys.getenv("SURVEYDOWN_PASSWORD"),
-    gssencmode = "prefer",
-    ignore     = FALSE,
-    min_size   = 1,
-    max_size   = Inf
+        host       = NULL,
+        dbname     = NULL,
+        port       = NULL,
+        user       = NULL,
+        table      = NULL,
+        password   = Sys.getenv("SURVEYDOWN_PASSWORD"),
+        gssencmode = "prefer",
+        ignore     = FALSE,
+        min_size   = 1,
+        max_size   = Inf
 ) {
 
     if (ignore) {
-        message("Database connection ignored. Saving data to local CSV file.")
+        message("Database connection ignored. Saving data to local CSV file.\n")
         return(NULL)
     }
 
@@ -166,11 +166,23 @@ sd_get_data <- function(db, refresh_interval = NULL) {
         warning("Database is not connected, db is NULL")
         return(NULL)
     }
+
     fetch_data <- function() {
+        # Check if table exists first
+        table_exists <- pool::poolWithTransaction(db$db, function(conn) {
+            DBI::dbExistsTable(conn, db$table)
+        })
+
+        if (!table_exists) {
+            return(NULL)
+        }
+
+        # Only try to read if table exists
         pool::poolWithTransaction(db$db, function(conn) {
             DBI::dbReadTable(conn, db$table)
         })
     }
+
     if (!is.null(refresh_interval)) {
         if (is.null(shiny::getDefaultReactiveDomain())) {
             stop('If refresh_interval is set to a positive number, sd_get_data() must be called within a reactive context for the data to continously update in the server.')
@@ -228,7 +240,7 @@ create_table <- function(data_list, db, table) {
         DBI::dbExecute(conn, paste0('ALTER TABLE "', table, '" ENABLE ROW LEVEL SECURITY;'))
     })
 
-    message(paste("Table", table, "created in the database."))
+    message(paste0('Table "', table, '" created in the database.'))
 }
 
 # Solution found in this issue:
@@ -250,8 +262,9 @@ sqlInterpolateList <- function(conn, sql, vars=list(), list_vars=list()) {
 }
 
 database_uploading <- function(data_list, db, table, changed_fields) {
-    if(is.null(db)) return(warning("Databasing is not in use"))
-
+    if(is.null(db)) {
+        return(warning("Databasing is not in use"))
+    }
     tryCatch({
         pool::poolWithTransaction(db, function(conn) {
             # Get the actual columns in the table
@@ -260,9 +273,11 @@ database_uploading <- function(data_list, db, table, changed_fields) {
             # Check for new fields
             new_fields <- setdiff(names(data_list), existing_cols)
             if (length(new_fields) > 0) {
+                # Add new fields to the table
                 for (field in new_fields) {
                     DBI::dbExecute(conn, sprintf('ALTER TABLE "%s" ADD COLUMN "%s" TEXT', table, field))
                 }
+                # Update existing_cols
                 existing_cols <- c(existing_cols, new_fields)
             }
 
@@ -270,20 +285,24 @@ database_uploading <- function(data_list, db, table, changed_fields) {
             data_list <- data_list[names(data_list) %in% c("session_id", intersect(changed_fields, existing_cols))]
 
             # If there's nothing to update (only session_id), return early
-            if (length(data_list) <= 1) return()
+            if (length(data_list) <= 1) {
+                return()
+            }
 
             # Ensure session_id is the first column
             cols <- c("session_id", setdiff(names(data_list), "session_id"))
             data_list <- data_list[cols]
 
-            # Prepare SQL components
+            # Prepare the placeholders
             placeholders <- paste0("?", names(data_list))
+
+            # Prepare the update set
             update_cols <- setdiff(cols, "session_id")
             update_set <- paste(sapply(update_cols, function(col) {
                 sprintf('"%s" = EXCLUDED."%s"', col, col)
             }), collapse = ", ")
 
-            # Build and execute query
+            # Prepare the SQL query template
             query_template <- sprintf(
                 'INSERT INTO "%s" ("%s") VALUES (%s) ON CONFLICT (session_id) DO UPDATE SET %s',
                 table,
@@ -292,10 +311,18 @@ database_uploading <- function(data_list, db, table, changed_fields) {
                 update_set
             )
 
-            query <- sqlInterpolateList(conn, query_template, list_vars = data_list)
+            # Use sqlInterpolateList to safely insert values
+            query <- sqlInterpolateList(
+                conn,
+                query_template,
+                list_vars = data_list
+            )
+
+            # Execute the query
             DBI::dbExecute(conn, query)
         })
     }, error = function(e) {
         warning("Error in database operation: ", e$message)
+        print(e)  # Print the full error for debugging
     })
 }
